﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Reflection;

namespace howto_reflection_list_events
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Make some events.
        public delegate int MyPublicDelegate(string index);
        public event MyPublicDelegate MyPublicEvent;

        private delegate int MyPrivateDelegate(string index);
        private event MyPrivateDelegate MyPrivateEvent;

        public virtual event MyPublicDelegate MyVirtualEvent;

        private void Form1_Load(object sender, EventArgs e)
        {
            // Make column headers.
            lvwEvents.Columns.Clear();
            lvwEvents.Columns.Add("Event", 200,
                HorizontalAlignment.Left);
            lvwEvents.Columns.Add("Attributes", 300,
                HorizontalAlignment.Left);
            lvwEvents.Columns.Add("Properties", 150,
                HorizontalAlignment.Left);

            // List the events.
            // Use the class you want to study instead of Form1.
            EventInfo[] event_infos = typeof(Form1).GetEvents(
                BindingFlags.FlattenHierarchy |
                BindingFlags.Instance |
                BindingFlags.NonPublic |
                BindingFlags.Public |
                BindingFlags.Static);
            foreach (EventInfo info in event_infos)
            {
                string properties = "";
                if (info.EventHandlerType.IsNotPublic) properties += " private";
                if (info.EventHandlerType.IsPublic) properties += " public";
                if (info.EventHandlerType.IsSealed) properties += " sealed";

                if (properties.Length > 0) properties = properties.Substring(1);
                ListViewMakeRow(lvwEvents,
                    info.Name,
                    info.EventHandlerType.Attributes.ToString(),
                    properties);
            }

            // Size the columns to fit the data.
            for (int i = 0; i < lvwEvents.Columns.Count; i++)
                lvwEvents.Columns[i].Width = 150;

            // Size the form.
            int new_wid = 30;
            for (int i = 0; i < lvwEvents.Columns.Count; i++)
                new_wid += lvwEvents.Columns[i].Width;
            this.Size = new Size(new_wid, this.Size.Height);
        }

        // Make a ListView row.
        private void ListViewMakeRow(ListView lvw, String item_title, params string[] subitem_titles)
        {
            // Make the item.
            ListViewItem new_item = lvw.Items.Add(item_title);

            // Make the sub-items.
            for (int i = subitem_titles.GetLowerBound(0); i <= subitem_titles.GetUpperBound(0); i++)
            {
                new_item.SubItems.Add(subitem_titles[i]);
            }
        }
    }
}
